 /*
  * Khoros: $Id: vfloor.c,v 1.1 1991/05/10 15:45:42 khoros Exp $
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id: vfloor.c,v 1.1 1991/05/10 15:45:42 khoros Exp $";
#endif

 /*
  * $Log: vfloor.c,v $
 * Revision 1.1  1991/05/10  15:45:42  khoros
 * Initial revision
 *
  */ 

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1991, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as too the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1991 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: vfloor.c
 >>>>
 >>>>      Program Name: vfloor
 >>>>
 >>>>      Program Desc: 
 >>>>	
 >>>>	Clip the lowest levels of an image to a constant value.
 >>>>	
 >>>>	
 >>>>            Author: Scott Wilson
 >>>>
 >>>> Date Last Updated: Wed Mar 27 23:57:28 1991
 >>>>
 >>>>          Routines: main- the main program for vfloor
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "vfloor.h"


/****************************************************************
 *
 * Routine Name:  main program for vfloor
 *
 *       Input:  
 *        -i  input image 
 *        -o  resulting output image 
 *        -l  floor level  
 *
 *
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */
struct xvimage *image,*readimage();
/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */
    if (check_args()) exit(1);
    image = readimage(vfloor->i_file);
    if (image == NULL) exit(1);               /* Quit if bad image */

    CHECKINPUT(program,image);  /* Verify input image */
/* -main_before_lib_call_end */

/* -main_library_call */
    if(! lvfloor(image,vfloor->l_float))
    {
        (void) fprintf(stderr, "lvfloor Failed\n");
        exit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
    writeimage(vfloor->o_file,image);
/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for vfloor 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "vfloor :\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\tClip the lowest levels of an image to a constant value.\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");

/* -usage_additions */
/* -usage_additions_end */

	fprintf(stderr,"\t-i  input image  (infile)\n");
	fprintf(stderr,"\t-o  resulting output image  (outfile)\n");
	fprintf(stderr,"\t-l  floor level  (float)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for vfloor\n");
	fprintf(stderr, "\t[-U] Gives the usage for vfloor\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called vfloor.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses vfloor.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for vfloor
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i_pstr = 
	"\nEnter: (i) input image \n        {infile}: "; 
char	*o_pstr = 
	"\nEnter: (o) resulting output image \n        {outfile}: "; 
char	*l_pstr = 
	"\nEnter: (l) floor level \n        {float}: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "vfloor", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "vfloor.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "vfloor: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "vfloor.ans");
		}

	     }

	}

	/*
	 * allocate the vfloor structure
	 */
	vfloor = (vfloor_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (vfloor_struct));

	/*
	 * get required arguments for vfloor
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(vfloor->i_flag = vget_infile(fid, prompt_flag,
			"-i", i_pstr, &(vfloor->i_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(vfloor->i_flag))
		{
		    fprintf(stderr, "\t\ti is required, please re-enter: \n");
		    vfloor->i_flag = vget_infile(fid, prompt_flag,
		    "-i", i_pstr, &(vfloor->i_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	if (!(vfloor->o_flag = vget_outfile(fid, prompt_flag,
			"-o", o_pstr, &(vfloor->o_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(vfloor->o_flag))
		{
		    fprintf(stderr, "\t\to is required, please re-enter: \n");
		    vfloor->o_flag = vget_outfile(fid, prompt_flag,
		    "-o", o_pstr, &(vfloor->o_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-o' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	if (!(vfloor->l_flag = vget_float(fid, prompt_flag,
			"-l", l_pstr, &(vfloor->l_float),
			0.000000, 0.000000, 0.000000)))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(vfloor->l_flag))
		{
		    fprintf(stderr, "\t\tl is required, please re-enter: \n");
		    vfloor->l_flag = vget_float(fid, prompt_flag,
		    "-l", l_pstr, &(vfloor->l_float),
		    0.000000, 0.000000, 0.000000);

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-l' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}


}



